#pragma once

#include <ctime>
#include <map>
#include <vector>
#include <string>
#include <fstream>
#include <sstream>

#include "boost/filesystem.hpp"
#include "boost/date_time/posix_time/posix_time.hpp"

namespace fs = boost::filesystem;
namespace t = boost::posix_time;

struct Trade {	// this is just a support struct used to speed up the initial data reads, don't use it any other places
	boost::posix_time::ptime time;
	double price;
	unsigned long volume;
};

struct AtomicCandle {
	time_t time;  // start of candle (duration is 1 min)
	double open;
	double high;
	double low;
	double close;
	unsigned long volume;
};

class DataMaker {
	std::string indir ;
	std::string outdir ;

	std::map< std::string , std::vector<Trade> > trades ;
	std::map< std::string , std::vector<AtomicCandle> > candles ;

	std::istringstream iss;	// for reading trades

	unsigned readTrades(std::ifstream& file) ;
	unsigned filterTrades(std::vector<Trade> &src) ;

	void read();		// read the trades
	void clean();		// filter the trade data
	void convert();		// convert to candlesticks
	void save();		// save the candlesticks

public:
	DataMaker( std::string _indir, std::string _outdir ) : indir(_indir), outdir(_outdir) { }
	void run() ;
} ;